# mysqldb_wrap.py
# Author: Angela Brooks
# Program Completion Date:
# Description: This is a wrapper class for the MySQLdb module.  I noticed that I
# us the same sort of functions with the module, so this will be a faster way
# to interface with a MySQL database
# Modification Date(s):
#
# class DB:
# 	__init__(self,host,user,passwd)
#	createDatabase(self,database_name)
#	createTable(self, table_name, create_definition, database)
#	insertIntoTable(self, table_name, column_names, values, database )
#	getDBRecords_Dict( self, select_statement, database)
#	getDBRow_Dict( self, select_statement, database)
# Copyright (c) 2011, Angela Brooks. anbrooks@gmail.com
# All rights reserved.

import sys, getopt, pdb

import MySQLdb

###########
#CONSTANTS#
###########

    
###############
#END CONSTANTS#
###############


#########
#CLASSES#
#########
class DB:

	def __init__(self, host, user, passwd):
		self.host = host
		self.user = user
		self.passwd = passwd

	def __connect(self, database):
		return MySQLdb.connect( host = self.host,
					user = self.user,
					passwd = self.passwd,
					db = database )	

	def createDatabase(self,database_name):
		conn = MySQLdb.connect( host = self.host,
					user = self.user,
					passwd = self.passwd )
		cursor = conn.cursor()
	
		#Delete previous database
		try:
			drop_statement = "DROP DATABASE %s" % database_name
			cursor.execute( drop_statement )
		except MySQLdb.Error, e:
			pass
	
		create_statement = "CREATE DATABASE %s" % database_name
		cursor.execute( create_statement )
		cursor.close()
		conn.close()

	def createTable(self, table_name, create_definition, database):
		conn = self.__connect( database )
		cursor = conn.cursor()
	
		#Delete previous table
		try:
			drop_statement = "DROP TABLE %s" % table_name
			cursor.execute( drop_statement )

		except MySQLdb.Error, e:
			pass

		create_statement = "CREATE TABLE %s (%s)" % (table_name,
							create_definition)
		cursor.execute( create_statement )
		cursor.close()
		conn.close()
	
	def insertIntoTable(self, table_name, column_names, values, database ):
		conn = self.__connect( database )
		cursor = conn.cursor()
		
		insert_statement = "INSERT INTO %s (%s) VALUES (%s)" % (table_name,
								  	column_names,
									values)	
		try:
			cursor.execute( insert_statement )
		except MySQLdb.Error, e:
			print "Insert Error %d: %s" % (e.args[0], e.args[1])
			
		cursor.close()
		conn.close()	

	def updateTable(self, table_name, update_statement, database):
		conn = self.__connect(database)
		cursor = conn.cursor()

		try:
			cursor.execute(update_statement)
		except MySQLdb.Error, e:
			print "Update Error %d: %s" % (e.args[0], e.args[1])

		cursor.close()
		conn.close()
	
	def getDBRecords_Dict( self, select_statement, database):
		"""
		Returns () if no records exist.
		"""
        	#Connect to Database
        	conn = self.__connect(database)

	        cursor = conn.cursor(MySQLdb.cursors.DictCursor)
		try:
			cursor.execute ( select_statement )
			return_result =  cursor.fetchall()
		
		except MySQLdb.Error, e:
			print "Get Records Error %d: %s" % (e.args[0], e.args[1])
			return None

        	#Close connections
        	cursor.close()
        	conn.close()

        	return return_result

	def getDBRow_Dict( self, select_statement, database):
		"""
		Returns None if no row exists
		"""
                #Connect to Database
		conn = self.__connect(database)

		cursor = conn.cursor(MySQLdb.cursors.DictCursor)

		try:
			cursor.execute ( select_statement )
			return_result =  cursor.fetchone()
		except MySQLdb.Error, e:
			print "Get Row Error %d: %s" % (e.args[0], e.args[1])
			return None
		
                cursor.close()
                conn.close()
                                                                                
                return return_result
	

#############
#END CLASSES#
#############
 
######
#MAIN#	
######
def main():		

	db = DB("localhost","root", "")

	db.createDatabase("lotsOstuff")

	db.createTable( "pens", "id INT AUTO_INCREMENT PRIMARY KEY,color VARCHAR(10)", "lotsOstuff")
	db.insertIntoTable( "pens", "color","\'blue\'", "lotsOstuff")
	db.insertIntoTable( "pens", "color", "\'red\'", "lotsOstuff")

	select_statement = "SELECT * FROM pens WHERE color=\'blue\'"	

	pen_record = db.getDBRow_Dict( select_statement, "lotsOstuff" )

	print "id=%d" % int(pen_record["id"])
	print "color=%s" % pen_record["color"]

	select_statement2 = "SELECT * FROM pens"
	
	pen_record = db.getDBRecords_Dict( select_statement2, "lotsOstuff")

	print "The pens are:"
	print pen_record

##########
#END_MAIN#
##########

###########
#FUNCTIONS#
###########
def formatLine( line ):
	#format line
	line = line.replace("\r","")
	line = line.replace("\n","")
	return line

###############
#END FUNCTIONS#	
###############	
if __name__ == "__main__": main()
