#!/usr/bin/env python
# scale_AS_exclusion_inclusion.py
# Author: Angela Brooks
# Program Completion Date:
# Modification Date(s):
# Copyright (c) 2011, Angela Brooks. anbrooks@gmail.com
# All rights reserved.

"""<One line description>.
"""

import sys
import optparse 
import pdb
#############
# CONSTANTS #
#############
INFINITY = 10000000000000000000000000

# Do not want to inflate counts
MIN_EXON_LEN = 100

BASE_NORM = 100
#################
# END CONSTANTS #
#################


###########
# CLASSES #
###########
class OptionParser(optparse.OptionParser):
    """
    Adding a method for required arguments.
    Taken from:
    http://www.python.org/doc/2.3/lib/optparse-extending-examples.html
    """
    def check_required(self, opt):
        option = self.get_option(opt)

        # Assumes the option's 'default' is set to None!
        if getattr(self.values, option.dest) is None:
            print "%s option not supplied" % option
            self.print_help()
            sys.exit(1)


###############
# END CLASSES #
###############
 
########
# MAIN #	
########
def main():
	
    opt_parser = OptionParser()
   
    # Add Options. Required options should have default=None
    opt_parser.add_option("--input",
                          dest="input_file",
                          type="string",
                          help="""Input file containing exclusion/inclusion
                                  counts. This should be the output to
                                  clusterASExons2.py""",
                          default=None)
    opt_parser.add_option("--total_counts",
                          dest="total_counts",
                          type="string",
                          help="""File containing total read counts for each
                                  sample.""",
                          default=None)
# I decided that this is not the best step to do the exon length normalization.
# This should be done in getASEventReadCounts
#   opt_parser.add_option("--normalize_exon_length",
#                         dest="exon_len_norm",
#                         action="store_true",
#                         help="""Will also normalize counts by per kilobase of
#                                 the exon lengths. Equivalent to the K of RPKM""",
#                         default=False)
    opt_parser.add_option("--output",
                          dest="output",
                          type="string",
                          help="""Output file with normalized
                                  exclusion/inclusion counts""",
                          default=None)

    (options, args) = opt_parser.parse_args()
	
    # validate the command line arguments
    opt_parser.check_required("--input")
    opt_parser.check_required("--total_counts")
    opt_parser.check_required("--output")

    input_file = open(options.input_file)
    total_count_file = open(options.total_counts)

    output_file = open(options.output, "w")

    sample2norm = getSampleCountRatio(total_count_file)
    total_count_file.close()

    exon_len_norm = False

    sample_order = None
    # Now normalize the table
    for line in input_file:
        line = formatLine(line)

        line_list = line.split("\t")

        if line.startswith("#"):
            sample_order = getSampleOrder(line_list)
            output_file.write(line + "\n")
            continue
    
        outline_list = line_list[:11]

        excl_exon_len = 0.0
        incl_exon_len = 0.0
        if exon_len_norm:
            excl_exon_len = getExonLenSum(line_list[7])
            incl_exon_len = getExonLenSum(line_list[8])

        outline_list.extend(normalizeCounts(sample_order,
                                       line_list[11:], 
                                       sample2norm,
                                       excl_exon_len,
                                       incl_exon_len))

        output_file.write("\t".join(outline_list) + "\n")

    input_file.close()
    output_file.close()
        
    sys.exit(0)

############
# END_MAIN #
############

#############
# FUNCTIONS #
#############
def formatLine(line):
    line = line.replace("\r","")
    line = line.replace("\n","")
    return line

def getExonLenSum(exon_coords):
    totalLen = 0.0
    if exon_coords == "":
        return 0.0
    exon_coordList1 = exon_coords.split(";")
    for exon_coord_group in exon_coordList1:
        exon_coordList2 = exon_coord_group.split(",")
        for exon_coord in exon_coordList2:
            chr, start_str, end_str = exon_coord.split("_")
            totalLen += (float(end_str) - float(start_str) + 1)

    return totalLen

def getSampleCountRatio(total_count_file):
    """ 
    Returns a dictionary:
    {sample:normalization_factor}
    """
    sample2norm = {}

    sample2count = {}

    min_count = INFINITY
    for line in total_count_file:
        line = formatLine(line)

        sample, count_str = line.split("\t")

        count = int(count_str)

        if count < min_count:
            min_count = count

        sample2count[sample] = float(count)

    # Normalization factor divides each count by the minimum
    for sample in sample2count:
        sample2norm[sample] = round(sample2count[sample]/min_count, 4)

    return sample2norm

def getSampleOrder(line_list):
    return line_list[11:]

def normalizeCounts(sample_order, excl_incl_counts, sample2norm,
                    excl_exon_len, incl_exon_len):
    """
    Returns normalized counts for the excl_incl_counts
    """
    return_list = []
    
    if len(sample_order) != len(excl_incl_counts):
        print "Problem with samples in input table, disagreeing with lane counts."

    for i in range(len(sample_order)):
        this_excl, this_incl = map(float, excl_incl_counts[i].split(";"))

        norm_excl = int(round(this_excl/sample2norm[sample_order[i]]))
        norm_incl = int(round(this_incl/sample2norm[sample_order[i]]))

        if excl_exon_len >= MIN_EXON_LEN:
            norm_excl = int(round(norm_excl/(excl_exon_len/BASE_NORM)))
        if incl_exon_len >= MIN_EXON_LEN:
            norm_incl = int(round(norm_incl/(incl_exon_len/BASE_NORM)))
            
        norm_excl_incl = "%d;%d" % (norm_excl, norm_incl)

        return_list.append(norm_excl_incl)

    return return_list 

#################
# END FUNCTIONS #	
#################	
if __name__ == "__main__": main()
